@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)hack.h	3.1	93/01/15	*/
+/*	SCCS Id: @(#)hack.h	3.2	96/05/09	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -9,13 +9,6 @@
 #include "config.h"
 #endif
 
-/* Some people comment out OVERLAY when uncommenting MOVERLAY */
-#if defined(MOVERLAY)
-# ifndef OVERLAY
-#  define OVERLAY
-# endif
-#endif
-
 /*	For debugging beta code.	*/
 #ifdef BETA
 #define Dpline	pline
@@ -27,12 +20,6 @@
 #define OFF		0
 #define BOLT_LIM    8	/* from this distance ranged attacks will be made */
 #define MAX_CARR_CAP	1000	/* so that boulders can be heavier */
-#ifndef __SASC_60
-#ifdef NULL
-#undef NULL
-#endif /* NULL */
-#define NULL  ((char *)0)
-#endif
 #define DUMMY { 0 }
 
 /* symbolic names for capacity levels */
@@ -43,22 +30,25 @@
 #define EXT_ENCUMBER	4
 #define OVERLOADED	5
 
-/* this is the way the game ends */
-/* if these are rearranged, the arrays in end.c will need to be changed */
+/*
+ * This is the way the game ends.  If these are rearranged, the arrays
+ * in end.c and topten.c will need to be changed.
+ */
 #define DIED		 0
 #define CHOKING		 1
 #define POISONING	 2
 #define STARVING	 3
 #define DROWNING	 4
 #define BURNING		 5
-#define CRUSHING	 6
-#define STONING		 7
-#define GENOCIDED	 8
-#define PANICKED	 9
-#define TRICKED		10
-#define QUIT		11
-#define ESCAPED		12
-#define ASCENDED	13
+#define DISSOLVED	 6
+#define CRUSHING	 7
+#define STONING		 8
+#define GENOCIDED	 9
+#define PANICKED	10
+#define TRICKED		11
+#define QUIT		12
+#define ESCAPED		13
+#define ASCENDED	14
 
 #ifndef DUNGEON_H	/* includes align.h */
 #include "dungeon.h"
@@ -73,10 +63,18 @@
 #ifndef OBJCLASS_H
 #include "objclass.h"
 #endif
-
+#ifndef YOUPROP_H
+#include "youprop.h"
+#endif
+#ifndef WINTYPE_H
+#include "wintype.h"
+#endif
 #ifndef DECL_H
 #include "decl.h"
 #endif
+#ifndef TIMEOUT_H
+#include "timeout.h"
+#endif
 
 NEARDATA extern coord bhitpos;	/* place where thrown weapon falls to the ground */
 
@@ -103,11 +101,7 @@ NEARDATA extern coord bhitpos;	/* place where thrown weapon falls to the ground
 #endif
 
 #ifndef DISPLAY_H
-#include  "display.h"
-#endif
-
-#ifndef WINTYPE_H
-#include  "wintype.h"
+#include "display.h"
 #endif
 
 #ifndef ENGRAVE_H
@@ -118,7 +112,7 @@ NEARDATA extern coord bhitpos;	/* place where thrown weapon falls to the ground
 #include "rect.h"
 #endif
 
-#ifdef OVERLAY	/* This doesn't belong here, but we have little choice */
+#ifdef USE_TRAMPOLI /* This doesn't belong here, but we have little choice */
 #undef NDECL
 #define NDECL(f) f()
 #endif
@@ -131,7 +125,7 @@ NEARDATA extern coord bhitpos;	/* place where thrown weapon falls to the ground
 #include "winprocs.h"
 #endif
 
-#if defined(OVERLAY) && !defined(MOVERLAY)
+#ifdef USE_TRAMPOLI
 #include "wintty.h"
 #undef WINTTY_H
 
@@ -141,10 +135,31 @@ NEARDATA extern coord bhitpos;	/* place where thrown weapon falls to the ground
 
 #undef EXTERN_H
 #include "extern.h"
-#endif /* OVERLAY */
+#endif /* USE_TRAMPOLI */
 
 #define NO_SPELL	0
 
+/* flags to control makemon() */
+#define NO_MM_FLAGS       0x00  /* use this rather than plain 0 */
+#define NO_MINVENT        0x01  /* suppress minvent when creating mon */
+
+/* flags to control query_objlist() */
+#define BY_NEXTHERE	  0x1	/* follow objlist by nexthere field */
+#define AUTOSELECT_SINGLE 0x2	/* if only 1 object, don't ask */
+#define USE_INVLET	  0x4	/* use object's invlet */
+#define INVORDER_SORT	  0x8	/* sort objects by packorder */
+#define SIGNAL_NOMENU	  0x10	/* return -1 rather than 0 if none allowed */
+
+/* Flags to control query_category() */
+/* BY_NEXTHERE used by query_category() too, so skip 0x01 */
+#define UNPAID_TYPES 0x02
+#define GOLD_TYPES   0x04
+#define WORN_TYPES   0x08
+#define ALL_TYPES    0x10
+#define BILLED_TYPES 0x20
+#define CHOOSE_ALL   0x40
+#define ALL_TYPES_SELECTED -2
+
 /*** some utility macros ***/
 #define yn(query) yn_function(query,ynchars, 'n')
 #define ynq(query) yn_function(query,ynqchars, 'q')
@@ -153,6 +168,28 @@ NEARDATA extern coord bhitpos;	/* place where thrown weapon falls to the ground
 #define nyNaq(query) yn_function(query,ynNaqchars, 'n')
 #define ynNaq(query) yn_function(query,ynNaqchars, 'y')
 
+/* Macros for scatter */
+#define VIS_EFFECTS  0x1	/* Display visual effects             */
+#define MAY_HITMON   0x2	/* Objects may hit monsters           */
+#define MAY_HITYOU   0x4	/* Objects may hit you                */
+#define MAY_HIT      0x6	/* Objects may hit you and monsters   */
+#define MAY_DESTROY  0x8	/* Objects may be destroyed at random */
+#define MAY_FRACTURE 0x10	/* Boulders & statues may fracture    */
+
+/* Macros for launching objects */
+#define ROLL	1
+#define FLING	2
+
+/* Flags to control menus */
+#define MENUTYPELEN sizeof("traditional ")
+#define MENU_TRADITIONAL 0
+#define MENU_COMBINATION 1
+#define MENU_PARTIAL	 2
+#define MENU_FULL	 3
+
+#define MENU_SELECTED	TRUE
+#define MENU_UNSELECTED	FALSE
+
 #ifndef max
 #define max(a,b) ((a) > (b) ? (a) : (b))
 #endif
@@ -170,23 +207,16 @@ NEARDATA extern coord bhitpos;	/* place where thrown weapon falls to the ground
 
 #define rn1(x,y)	(rn2(x)+(y))
 
-#ifndef MUSE
-#define find_mac(m)	((m)->data->ac)
-#endif
+/* negative armor class is randomly weakened to prevent invulnerability */
+#define AC_VALUE(AC)	((AC) >= 0 ? (AC) : -rnd(-(AC)))
 
-#if defined(MICRO)
+#if defined(MICRO) && !defined(__DJGPP__)
 #define getuid() 1
-#define getlogin() (NULL)
+#define getlogin() ((char *)0)
 #endif /* MICRO */
 
-/* Macro for a few items that are only static if we're not overlaid.... */
-#if defined(OVERLAY)
-# define STATIC_PTR
-#else
-# define STATIC_PTR static
-#endif
-
 #if defined(OVERLAY)&&(defined(OVL0)||defined(OVL1)||defined(OVL2)||defined(OVL3)||defined(OVLB))
+# define USE_OVLx
 # define STATIC_DCL extern
 # define STATIC_OVL
 # ifdef OVLB
@@ -208,4 +238,11 @@ NEARDATA extern coord bhitpos;	/* place where thrown weapon falls to the ground
 # define OVLB	/* The base overlay segment */
 #endif	/* OVERLAY && (OVL0 || OVL1 || OVL2 || OVL3 || OVLB) */
 
+/* Macro for a few items that are only static if we're not overlaid.... */
+#if defined(USE_TRAMPOLI) || defined(USE_OVLx)
+# define STATIC_PTR
+#else
+# define STATIC_PTR static
+#endif
+
 #endif /* HACK_H */
