@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)vault.h	3.1	92/04/18	*/
+/*	SCCS Id: @(#)vault.h	3.2	92/04/18	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -10,15 +10,17 @@ struct fakecorridor {
 	xchar fx,fy,ftyp;
 };
 
+#ifndef DUNGEON_H
 #include "dungeon.h"
+#endif
 
 struct egd {
 	int fcbeg, fcend;	/* fcend: first unused pos */
+	int vroom;		/* room number of the vault */
 	xchar gdx, gdy;		/* goal of guard's walk */
 	xchar ogx, ogy;		/* guard's last position */
 	d_level gdlevel;	/* level (& dungeon) guard was created in */
 	xchar warncnt;		/* number of warnings to follow */
-	int vroom;		/* room number of the vault */
 	Bitfield(gddone,1);	/* true iff guard has released player */
 	Bitfield(unused,7);
 	struct fakecorridor fakecorr[FCSIZ];
