@@ -1,27 +1,30 @@
-/*	SCCS Id: @(#)track.c	3.1	87/08/08
+/*	SCCS Id: @(#)track.c	3.2	87/08/08	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 /* track.c - version 1.0.2 */
 
 #include "hack.h"
 
-#define	UTSZ	50
+#define UTSZ	50
 
 STATIC_VAR NEARDATA int utcnt, utpnt;
 STATIC_VAR NEARDATA coord utrack[UTSZ];
 
 #ifdef OVLB
+
 void
-initrack(){
+initrack()
+{
 	utcnt = utpnt = 0;
 }
-#endif /* OVLB */
 
+#endif /* OVLB */
 #ifdef OVL1
 
 /* add to track */
 void
-settrack(){
+settrack()
+{
 	if(utcnt < UTSZ) utcnt++;
 	if(utpnt == UTSZ) utpnt = 0;
 	utrack[utpnt].x = u.ux;
