@@ -1,17 +1,17 @@
-/*	SCCS Id: @(#)macunix.c	3.1	90/22/02
+/*	SCCS Id: @(#)macunix.c	3.1	94/11/07	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
 /* This file collects some Unix dependencies */
 
 #include "hack.h"
+
+#ifndef __MWERKS__
 #include <fcntl.h>
-
-int FDECL(uptodate, (int));
-void FDECL(regularize,(char *));
-void NDECL(getlock);
+#endif
 
 
+#if 0
 int
 uptodate(int fd)
 {
@@ -20,6 +20,7 @@ uptodate(int fd)
 #endif
 	return(1);
 }
+#endif
 
 
 void
