@@ -8,7 +8,7 @@ static char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
 #define yyerrok (yyerrflag=0)
 #define YYRECOVERING (yyerrflag!=0)
 #define YYPREFIX "yy"
-/*	SCCS Id: @(#)dgn_comp.c	3.2	96/05/10	*/
+/*	SCCS Id: @(#)dgn_comp.c	3.2	96/06/22	*/
 /*	Copyright (c) 1989 by Jean-Christophe Collet */
 /*	Copyright (c) 1990 by M. Stephenson				  */
 /* NetHack may be freely redistributed.  See license for details. */
@@ -57,13 +57,6 @@ void NDECL(output_dgn);
 #endif
 #endif
 
-#ifdef MICRO
-# undef exit
-# if !defined(MSDOS) && !defined(WIN32)
-extern void FDECL(exit, (int));
-# endif
-#endif
-
 #define ERR		(-1)
 
 static struct couple couple;
@@ -548,7 +541,7 @@ output_dgn()
 {
 	int	nd, cl = 0, nl = 0,
 		    cb = 0, nb = 0;
-	static unsigned long version_info[4] = {
+	static struct version_info version_data = {
 			VERSION_NUMBER, VERSION_FEATURES,
 			VERSION_SANITY1, VERSION_SANITY2
 	};
@@ -558,7 +551,7 @@ output_dgn()
 	    exit(EXIT_FAILURE);
 	}
 
-	if (fwrite((char *)version_info, sizeof version_info, 1, yyout) != 1) {
+	if (fwrite((char *)&version_data, sizeof version_data, 1, yyout) != 1) {
 	    yyerror("FATAL - output failure.");
 	    exit(EXIT_FAILURE);
 	}
