@@ -7,6 +7,7 @@
 #include "hack.h"
 #include "dlb.h"
 
+#include <sys/stat.h>
 #include <signal.h>
 #include <pwd.h>
 #ifndef O_RDONLY
@@ -214,9 +215,9 @@ char *argv[];
 		(void) chmod(SAVEF,0);	/* disallow parallel restores */
 		(void) signal(SIGINT, (SIG_RET_TYPE) done1);
 #ifdef NEWS
-		if(flags.news) {
+		if(iflags.news) {
 		    display_file(NEWS, FALSE);
-		    flags.news = FALSE; /* in case dorecover() fails */
+		    iflags.news = FALSE; /* in case dorecover() fails */
 		}
 #endif
 		pline("Restoring save file...");
@@ -306,7 +307,7 @@ char *argv[];
 			break;
 #ifdef NEWS
 		case 'n':
-			flags.news = FALSE;
+			iflags.news = FALSE;
 			break;
 #endif
 		case 'u':
