@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)panic.c	3.1	93/01/07	*/
+/*	SCCS Id: @(#)panic.c	3.2	94/03/02	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -8,19 +8,15 @@
  */
 
 #define NEED_VARARGS
-#include	"config.h"
+#include "config.h"
 
-#ifdef MICRO
-#undef exit
-extern void FDECL(exit, (int));
-#endif
 #ifdef AZTEC
 #define abort() exit()
 #endif
 #ifdef VMS
 extern void NDECL(vms_abort);
 #endif
- 
+
 /*VARARGS1*/
 boolean panicking;
 void VDECL(panic, (char *,...));
@@ -45,29 +41,20 @@ panic VA_DECL(char *,str)
 		    abort();	/* generate core dump */
 #endif
 	VA_END();
-	exit(1);		/* redundant */
+	exit(EXIT_FAILURE);		/* redundant */
 	return;
 }
 
 #ifdef ALLOCA_HACK
-extern long *alloc();
 /*
- * In case bison generated foo_yacc.c tries to use alloca(); if we don't
+ * In case bison-generated foo_yacc.c tries to use alloca(); if we don't
  * have it then just use malloc() instead.  This may not work on some
  * systems, but they should either use yacc or get a real alloca routine.
  */
 long *alloca(cnt)
 unsigned cnt;
 {
-	static long dummy;
-	/*
-	 * Assumption:  result returned by alloca() will never be freed.
-	 *	alloca(0) may be used as a trigger to release memory; we cant
-	 *	oblige it, and we don't want to attempt to allocate 0 bytes
-	 *	or return a NULL pointer, so return pointer to `dummy' rather
-	 *	than allocating at least one byte.
-	 */
-	return cnt ? alloc(cnt) : &dummy;
+	return cnt ? alloc(cnt) : (long *)0;
 }
 #endif
 
