@@ -1,11 +1,11 @@
-/*	SCCS Id: @(#)wintext.c	3.1	92/3/7
+/*	SCCS Id: @(#)wintext.c	3.2	96/04/05	*/
 /* Copyright (c) Dean Luick, 1992				  */
 /* NetHack may be freely redistributed.  See license for details. */
 
 /*
  * File for dealing with text windows.
- * 
- * 	+ No global functions.
+ *
+ *	+ No global functions.
  */
 
 #ifndef SYSV
@@ -16,6 +16,7 @@
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
 #include <X11/Xos.h>
+#include <X11/Xaw/Form.h>
 #include <X11/Xaw/AsciiText.h>
 #include <X11/Xaw/Cardinals.h>
 #include <X11/Xatom.h>
@@ -29,6 +30,11 @@
 
 #include "hack.h"
 #include "winX.h"
+#include "Window.h"
+
+#ifdef GRAPHIC_TOMBSTONE
+#include <X11/xpm.h>
+#endif
 
 
 #define TRANSIENT_TEXT	/* text window is a transient window (no positioning) */
@@ -38,6 +44,14 @@ static const char text_translations[] =
      <BtnDown>: dismiss_text()\n\
      <Key>: key_dismiss_text()";
 
+#ifdef GRAPHIC_TOMBSTONE
+static const char rip_translations[] =
+    "#override\n\
+     <BtnDown>: rip_dismiss_text()\n\
+     <Key>: rip_dismiss_text()";
+
+static Widget FDECL(create_ripout_widget, (Widget));
+#endif
 
 /*ARGSUSED*/
 void
@@ -53,7 +67,7 @@ delete_text(w, event, params, num_params)
     wp = find_widget(w);
     text_info = wp->text_information;
 
-    nh_XtPopdown(w);
+    nh_XtPopdown(wp->popup);
 
     if (text_info->blocked) {
 	exit_x_event = TRUE;
@@ -77,12 +91,11 @@ dismiss_text(w, event, params, num_params)
 {
     struct xwindow *wp;
     struct text_info_t *text_info;
-    Widget popup = XtParent(w);
 
     wp = find_widget(w);
     text_info = wp->text_information;
 
-    nh_XtPopdown(popup);
+    nh_XtPopdown(wp->popup);
 
     if (text_info->blocked) {
 	exit_x_event = TRUE;
@@ -103,6 +116,20 @@ key_dismiss_text(w, event, params, num_params)
     if (ch) dismiss_text(w, event, params, num_params);
 }
 
+#ifdef GRAPHIC_TOMBSTONE
+/* Dismiss from clicking on rip image. */
+void
+rip_dismiss_text(w, event, params, num_params)
+    Widget w;
+    XEvent *event;
+    String *params;
+    Cardinal *num_params;
+{
+    dismiss_text(XtParent(w), event, params, num_params);
+}
+#endif
+
+
 /* ARGSUSED */
 void
 add_to_text_window(wp, attr, str)
@@ -148,10 +175,10 @@ display_text_window(wp, blocking)
     nlines = (XtScreen(wp->w)->height - text_info->extra_height) /
 			(text_info->fs->ascent + text_info->fs->descent);
     nlines -= 4;
-    if (nlines <= 0) nlines = 1;
 
     if (nlines > text_info->text.num_lines)
 	nlines = text_info->text.num_lines;
+    if (nlines <= 0) nlines = 1;
 
     /* Font height is ascent + descent. */
     height = (nlines * (text_info->fs->ascent + text_info->fs->descent))
@@ -166,6 +193,13 @@ display_text_window(wp, blocking)
 	width += 20;
     }
 
+#ifdef GRAPHIC_TOMBSTONE
+    if (text_info->is_rip) {
+	Widget rip = create_ripout_widget(XtParent(wp->w));
+	XtSetArg(args[num_args], XtNfromVert, rip);	num_args++;
+    }
+#endif
+
     if (width > (Dimension) XtScreen(wp->w)->width) { /* too wide for screen */
 	/* Back off some amount - we really need to back off the scrollbar */
 	/* width plus some extra.					   */
@@ -178,7 +212,7 @@ display_text_window(wp, blocking)
 
 #ifdef TRANSIENT_TEXT
     XtRealizeWidget(wp->popup);
-    XSetWMProtocols(XtDisplay(wp->popup), XtWindow(wp->popup), 
+    XSetWMProtocols(XtDisplay(wp->popup), XtWindow(wp->popup),
 		    &wm_delete_window, 1);
     positionpopup(wp->popup, FALSE);
 #endif
@@ -214,10 +248,11 @@ create_text_window(wp)
     Arg args[8];
     Cardinal num_args;
     Position top_margin, bottom_margin, left_margin, right_margin;
+    Widget form;
 
     wp->type = NHW_TEXT;
 
-    wp->text_information = text_info = 
+    wp->text_information = text_info =
 		    (struct text_info_t *) alloc(sizeof(struct text_info_t));
 
     init_text_buffer(&text_info->text);
@@ -226,9 +261,14 @@ create_text_window(wp)
     text_info->extra_height   = 0;
     text_info->blocked	      = FALSE;
     text_info->destroy_on_ack = TRUE;	/* Ok to destroy before display */
+#ifdef GRAPHIC_TOMBSTONE
+    text_info->is_rip	      = FALSE;
+#endif
 
     num_args = 0;
     XtSetArg(args[num_args], XtNallowShellResize, True); num_args++;
+    XtSetArg(args[num_args], XtNtranslations,
+		XtParseTranslationTable(text_translations));	num_args++;
 
 #ifdef TRANSIENT_TEXT
     wp->popup = XtCreatePopupShell("text", transientShellWidgetClass,
@@ -240,6 +280,11 @@ create_text_window(wp)
     XtOverrideTranslations(wp->popup,
 	XtParseTranslationTable("<Message>WM_PROTOCOLS: delete_text()"));
 
+    num_args = 0;
+    XtSetArg(args[num_args], XtNallowShellResize, True);	num_args++;
+    form = XtCreateManagedWidget("form", formWidgetClass, wp->popup,
+		args, num_args);
+
     num_args = 0;
     XtSetArg(args[num_args], XtNdisplayCaret, False);		num_args++;
     XtSetArg(args[num_args], XtNresize, XawtextResizeBoth);	num_args++;
@@ -250,7 +295,7 @@ create_text_window(wp)
 		killer && WIN_MAP == WIN_ERR ?
 				  "tombstone" : "text_text", /* name */
 		asciiTextWidgetClass,
-		wp->popup,		/* parent widget */
+		form,			/* parent widget */
 		args,			/* set some values */
 		num_args);		/* number of values to set */
 
@@ -284,13 +329,20 @@ destroy_text_window(wp)
     if (text_info->blocked || text_info->destroy_on_ack) {
 	XtDestroyWidget(wp->popup);
 	free_text_buffer(&text_info->text);
-	free((char *) text_info);
+	free((genericptr_t)text_info),  wp->text_information = 0;
 	wp->type = NHW_NONE;	/* allow reuse */
     } else {
 	text_info->destroy_on_ack = TRUE;	/* destroy on next ACK */
     }
 }
 
+void
+clear_text_window(wp)
+    struct xwindow *wp;
+{
+    clear_text_buffer(&wp->text_information->text);
+}
+
 
 /* text buffer routines ---------------------------------------------------- */
 
@@ -318,7 +370,7 @@ append_text_buffer(tb, str, concat)
 	printf("append_text_buffer: text buffer growing from %d to %d bytes\n",
 				tb->text_size, 2*tb->text_size);
 #endif
-	copy = (char *) alloc(tb->text_size*2);
+	copy = (char *) alloc((unsigned)tb->text_size*2);
 	(void) memcpy(copy, tb->text, tb->text_last);
 	free(tb->text);
 	tb->text = copy;
@@ -342,7 +394,7 @@ append_text_buffer(tb, str, concat)
 	if(length) {
 	    /* Remove all newlines. Otherwise we have a confused line count. */
 	    copy = (tb->text+tb->text_last);
-	    while (copy = index(copy, '\n'))
+	    while ((copy = index(copy, '\n')) != (char*)0)
 		*copy = ' ';
 	}
 
@@ -385,3 +437,188 @@ free_text_buffer(tb)
     tb->text_last = 0;
     tb->num_lines = 0;
 }
+
+
+#ifdef GRAPHIC_TOMBSTONE
+
+static void FDECL(rip_exposed, (Widget,XtPointer,XtPointer));
+
+static XImage* rip_image=0;
+
+
+#define STONE_LINE_LEN 16	/* # chars that fit on one line */
+#define NAME_LINE 0		/* line # for player name */
+#define GOLD_LINE 1		/* line # for amount of gold */
+#define DEATH_LINE 2		/* line # for death description */
+#define YEAR_LINE 6		/* line # for year */
+
+static char rip_line[YEAR_LINE+1][STONE_LINE_LEN+1];
+
+extern const char *killed_by_prefix[];
+
+void
+calculate_rip_text(int how)
+{
+	/* Follows same algorithm as genl_outrip() */
+
+	char buf[BUFSZ];
+	char *dpx;
+	int line;
+
+	/* Put name on stone */
+	Sprintf(rip_line[NAME_LINE], "%s", plname);
+
+	/* Put $ on stone */
+	Sprintf(rip_line[GOLD_LINE], "%ld Au", u.ugold);
+
+	/* Put together death description */
+	switch (killer_format) {
+		default: impossible("bad killer format?");
+		case KILLED_BY_AN:
+			Strcpy(buf, killed_by_prefix[how]);
+			Strcat(buf, an(killer));
+			break;
+		case KILLED_BY:
+			Strcpy(buf, killed_by_prefix[how]);
+			Strcat(buf, killer);
+			break;
+		case NO_KILLER_PREFIX:
+			Strcpy(buf, killer);
+			break;
+	}
+
+	/* Put death type on stone */
+	for (line=DEATH_LINE, dpx = buf; line<YEAR_LINE; line++) {
+		register int i,i0;
+		char tmpchar;
+
+		if ( (i0=strlen(dpx)) > STONE_LINE_LEN) {
+			for(i = STONE_LINE_LEN;
+			    ((i0 > STONE_LINE_LEN) && i); i--)
+				if(dpx[i] == ' ') i0 = i;
+			if(!i) i0 = STONE_LINE_LEN;
+		}
+		tmpchar = dpx[i0];
+		dpx[i0] = 0;
+		strcpy(rip_line[line], dpx);
+		if (tmpchar != ' ') {
+			dpx[i0] = tmpchar;
+			dpx= &dpx[i0];
+		} else  dpx= &dpx[i0+1];
+	}
+
+	/* Put year on stone */
+	Sprintf(rip_line[YEAR_LINE], "%4d", getyear());
+}
+
+
+/*
+ * RIP image expose callback.
+ */
+/*ARGSUSED*/
+static void
+rip_exposed(w, client_data, widget_data)
+    Widget w;
+    XtPointer client_data;	/* unused */
+    XtPointer widget_data;	/* expose event from Window widget */
+{
+    XExposeEvent *event = (XExposeEvent *) widget_data;
+    Display* dpy=XtDisplay(w);
+    Arg args[8];
+    XGCValues values;
+    XtGCMask mask;
+    GC gc;
+    static Pixmap rip_pixmap=None;
+    int i, x, y;
+
+    if (!XtIsRealized(w) || event->count > 0) return;
+
+    if (rip_pixmap == None && rip_image) {
+	rip_pixmap = XCreatePixmap(dpy, XtWindow(w),
+			rip_image->width,
+			rip_image->height,
+			DefaultDepth(dpy, DefaultScreen(dpy)));
+	XPutImage(dpy, rip_pixmap,
+		DefaultGC(dpy, DefaultScreen(dpy)),
+		rip_image,
+		0,0, 0,0,		/* src, dest top left */
+		rip_image->width,
+		rip_image->height);
+	XDestroyImage(rip_image);	/* data bytes free'd also */
+    }
+
+    mask = GCFunction | GCForeground | GCGraphicsExposures | GCFont;
+    values.graphics_exposures = False;
+    XtSetArg(args[0], XtNforeground, &values.foreground);
+    XtGetValues(w, args, 1);
+    values.function = GXcopy;
+    values.font = WindowFont(w);
+    gc = XtGetGC(w, mask, &values);
+
+    if (rip_pixmap != None) {
+	XCopyArea(dpy, rip_pixmap, XtWindow(w), gc,
+		event->x, event->y, event->width, event->height,
+		event->x, event->y);
+    }
+
+    x=appResources.tombtext_x;
+    y=appResources.tombtext_y;
+    for (i=0; i<=YEAR_LINE; i++) {
+	int len=strlen(rip_line[i]);
+	XFontStruct* font=WindowFontStruct(w);
+	int width=XTextWidth(font, rip_line[i], len);
+	XDrawString(dpy, XtWindow(w), gc,
+		x-width/2, y, rip_line[i], len);
+	x+=appResources.tombtext_dx;
+	y+=appResources.tombtext_dy;
+    }
+
+    XtReleaseGC(w, gc);
+}
+
+/*
+ * The ripout window creation routine.
+ */
+static Widget
+create_ripout_widget(Widget parent)
+{
+    Widget imageport;
+    Arg args[16];
+    Cardinal num_args;
+
+    static int rip_width, rip_height;
+
+    if (!rip_image) {
+	XpmAttributes attributes;
+	int errorcode;
+
+	attributes.valuemask = XpmCloseness;
+	attributes.closeness = 65535; /* Try anything */
+	errorcode = XpmReadFileToImage(XtDisplay(parent), appResources.tombstone, &rip_image, 0, &attributes);
+	if (errorcode != XpmSuccess) {
+	    char buf[BUFSIZ];
+	    Sprintf(buf, "Failed to load %s: %s", appResources.tombstone,
+			XpmGetErrorString(errorcode));
+	    X11_raw_print(buf);
+	}
+	rip_width = rip_image->width;
+	rip_height = rip_image->height;
+    }
+
+    num_args = 0;
+    XtSetArg(args[num_args], XtNwidth, rip_width); num_args++;
+    XtSetArg(args[num_args], XtNheight, rip_height); num_args++;
+    XtSetArg(args[num_args], XtNtranslations,
+		XtParseTranslationTable(rip_translations));	num_args++;
+
+    imageport = XtCreateManagedWidget("rip", windowWidgetClass,
+		parent, args, num_args);
+
+    XtAddCallback(imageport, XtNexposeCallback, rip_exposed, (XtPointer) 0);
+
+    return imageport;
+}
+
+#endif /* GRAPHIC_TOMBSTONE */
+
+/*wintext.c*/
